from __future__ import absolute_import, division, print_function
import inspect
import sys


def lineno():
    """Returns the current line number in our program."""
    return str(' - RuleDefinition - line number: '+str(inspect.currentframe().f_back.f_lineno))


class RuleDefinition:
    """
    Rule definition
    """

    def __init__(self, id, type,message, debug=False):
        """
        Initialize RuleDefinition
        :param id: 
        :param type: 
        :param message: 
        :param debug: 
        """
        self.id = id
        self.type = type
        self.message = message
        self.debug= debug

        if self.debug:
            print('RuleDefinition - init'+lineno())
            print('id: '+str(id)+lineno())
            print('type: '+str(type)+lineno())
            print('message: '+str(message)+lineno())
            print('debug: '+str(debug)+lineno())

    def to_string(self):
        """
        Convert rule definition to a string
        :return: 
        """
        if self.debug:
            print('to_string'+lineno())

        return "id: "+str(self.id)+", type: "+str(self.type)+", message: "+str(self.message)

    def to_hash(self):
        """
        Convert RuleDefinition to a hash
        :return: 
        """
        if self.debug:
            print('to_hash'+lineno())
            print('id: '+str(self.id)+lineno())

        data = {
            'id': str(self.id),
            'type': str(self.type),
            'message': str(self.message)
        }

        return data





from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule


def lineno():
    """Returns the current line number in our program."""
    return str(' - SnsTopicPolicyNotPrincipalRule - caller: ' + str(inspect.stack()[1][3]) + ' - line number: ' + str(
        inspect.currentframe().f_back.f_lineno))


class SnsTopicPolicyNotPrincipalRule(BaseRule):

    def __init__(self, cfn_model=None, debug=None):
        '''
        Initialize
        :param cfn_model:
        '''
        BaseRule.__init__(self, cfn_model, debug=debug)

    def rule_text(self):
        '''
        Get rule text
        :return:
        '''
        if self.debug:
            print('rule_text' + lineno())
        return 'SNS Topic policy should not allow Allow+NotPrincipal'

    def rule_type(self):
        '''
        Get rule type
        :return:
        '''
        self.type = 'VIOLATION::FAILING_VIOLATION'
        return 'VIOLATION::FAILING_VIOLATION'

    def rule_id(self):
        '''
        Get rule id
        :return:
        '''
        if self.debug:
            print('rule_id' + lineno())
        self.id = 'F8'
        return 'F8'

    def audit_impl(self):
        '''
        Audit
        :return: violations
        '''
        if self.debug:
            print('SnsTopicPolicyNotPrincipalRule - audit_impl' + lineno())

        violating_policies = []
        resources = self.cfn_model.resources_by_type('AWS::SNS::TopicPolicy')

        if len(resources) > 0:
            for resource in resources:
                if self.debug:
                    print('resource: ' + str(resource) + lineno())
                    print('vars: ' + str(vars(resource)) + lineno())

                if hasattr(resource, 'policy_document'):

                    if resource.policy_document:

                        if resource.policy_document.allows_not_principal():
                            violating_policies.append(str(resource.logical_resource_id))

        else:
            if self.debug:
                print('no violating_policies' + lineno())

        if self.debug:
            input('Press enter to continue: ' + lineno())

        return violating_policies


from __future__ import absolute_import, division, print_function
import inspect
import sys


def lineno():
    """Returns the current line number in our program."""
    return str(' - SimpleStdoutResults - line number: '+str(inspect.currentframe().f_back.f_lineno))

class SimpleStdoutResults:
    """
    Simple standard out results
    """
    def __init__(self):
        """
        Initialize SimpleStdoutResults
        """
        print('SimpleStdoutResults - init'+lineno())

    def message_violations(self, violations):
        print('message_violations'+lineno())
        # FIXME
        sys.exit(1)

    def render(self, results):
        """
        Render results
        :param results:
        :return:
        """
        # FIXME
        sys.exit(1)
        return('render'+lineno())

    def message(self, message_type,  message, logical_resource_ids=None):
        """
        Does something with message
        :param message_type:
        :param message:
        :param logical_resource_ids:
        :return:
        """
        print('message'+lineno())
        # FIXME
        sys.exit(1)


from __future__ import absolute_import, division, print_function
import inspect
import sys
import json
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule

def lineno():
  """Returns the current line number in our program."""
  return str(' -  CloudFormationAuthenticationRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))


class CloudFormationAuthenticationRule(BaseRule):

    def __init__(self, cfn_model=None, debug=None):
          """
          Initialize CloudFormationAuthenticationRule
          :param cfn_model:
          """
          BaseRule.__init__(self, cfn_model, debug=debug)

    def rule_text(self):
          """
          Returns rule text
          :return:
          """
          if self.debug:
            print('rule_text'+lineno())
          return 'Specifying credentials in the template itself is probably not the safest thing'


    def rule_type(self):
          """
          Returns the rule type
          :return:
          """
          self.type= 'VIOLATION::WARNING'
          return 'VIOLATION::WARNING'

    def rule_id(self):
          """
          Returns the rule_id
          :return:
          """
          if self.debug:
                print('rule_id'+lineno())
          self.id = 'W1'
          return 'W1'

    def audit_impl(self):
        """
        Audit
        :return: violations
        """
        if self.debug:
            print('CloudFormationAuthnticationRule - audit_impl'+lineno())
            print('model: '+str(self.cfn_model.raw_model)+lineno())
            print('model type: '+str(type(self.cfn_model.raw_model))+lineno())

        logical_resource_ids = []

        if type(self.cfn_model.raw_model) == type(str()):
            json_acceptable_string = self.cfn_model.raw_model.replace("'", "\"")
            self.cfn_model.raw_model= json.loads(json_acceptable_string)



        # Get the resource type
        for resource in self.cfn_model.raw_model['resource']:
            if self.debug:
                print('resource: '+str(resource)+lineno())


            for resource_name in self.cfn_model.raw_model['resource'][resource]:
                if 'metadata' in self.cfn_model.raw_model['resource'][resource][resource_name]:
                    if self.debug:
                        print('found metadata')

                    if 'AWS::CloudFormation::Authentication' in self.cfn_model.raw_model['resource'][resource][resource_name]['metadata']:
                        if self.debug:
                            print('has authentication')

                        if self.potentially_sensitive_credentials(self.cfn_model.raw_model['Resources'][resource][resource_name]['metadata']['AWS::CloudFormation::Authentication']):
                            logical_resource_ids.append(str(resource))

        if self.debug:
            input('Press enter to continue: ' + lineno())

        return logical_resource_ids

    def potentially_sensitive_credentials(self, auth):
        """
        Whether there are potentially sensitive credentials
        :param auth:
        :return:
        """
        if self.debug:
            print('potentially sensitive credentials'+lineno())

        # Example
        # "Metadata": {
        #   "AWS::CloudFormation::Authentication": {
        #     "testBasic" : {
        #       "type" : "basic",
        #       "username" : "biff",
        #       "password" : "badpassword",
        #       "uris" : [ "http://www.example.com/test" ]
        #     }
        #   }
        # }

        if type(auth) == type(dict()):
            for item in auth:

                if type(auth[item]) == type(dict()):

                    for item2 in auth[item]:
                        if self.debug:
                            print('item: '+str(item2))
                        if str(item2) in ['accessKeyId','password','secretKey']:
                            return True

        return False


from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule

def lineno():
    """Returns the current line number in our program."""
    return str(' - IamRoleWildcardActionOnTrustPolicyRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))

class IamRoleWildcardActionOnTrustPolicyRule(BaseRule):

  def __init__(self, cfn_model=None, debug=None):
    """
    Initialize
    :param cfn_model: 
    """
    BaseRule.__init__(self, cfn_model,debug=debug)
      
  def rule_text(self):
    """
    Return rule text
    :return: 
    """
    if self.debug:
      print('rule_text'+lineno())
    return 'IAM role should not allow * action on its trust policy'


  def rule_type(self):
    """
    Get rule type
    :return: 
    """
    self.type= 'VIOLATION::FAILING_VIOLATION'
    return 'VIOLATION::FAILING_VIOLATION'


  def rule_id(self):
    """
    Get rule id
    :return: 
    """
    if self.debug:
      print('rule_id'+lineno())
    self.id ='F2'
    return 'F2'


  def audit_impl(self):
    """
    Audit
    :return: violations 
    """
    if self.debug:
      print('IamRoleWildcardActionOnTrustPolicyRule - audit_impl'+lineno())

    violating_roles = []

    resources = self.cfn_model.resources_by_type('AWS::IAM::Role')

    if len(resources)>0:
      for resource in resources:
        if self.debug:
          print('resource: '+str(resource)+lineno())
          print('vars: '+str(vars(resource))+lineno())

        if hasattr(resource, 'assume_role_policy_document'):
          if self.debug:
            print('has assume role policy document ' + lineno())

          if resource.assume_role_policy_document:
            if self.debug:
              print('policy document: '+str(resource.assume_role_policy_document)+lineno())
              print('vars: '+str(resource.assume_role_policy_document)+lineno())


            if resource.assume_role_policy_document.wildcard_allowed_actions():
              if self.debug:
                print('has wildcard allowed actions')

              violating_roles.append(str(resource.logical_resource_id))
            else:
              if self.debug:
                print('does not have wildcare allowed actions')
    else:
      if self.debug:
        print('no violating_roles'+lineno())

    if self.debug:
      input('Press enter to continue: '+lineno())

    return violating_roles


from __future__ import absolute_import, division, print_function
import sys
import inspect
from builtins import (str)
from terraform_validator.Violation import Violation
from terraform_validator.custom_rules.BaseRule import BaseRule

def lineno():
    """Returns the current line number in our program."""
    return str(' - IamRoleNotResourceOnPermissionsPolicyRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))


class IamRoleNotResourceOnPermissionsPolicyRule(BaseRule):

  def __init__(self, cfn_model=None, debug=None):
    """
    Initialize
    :param cfn_model: 
    """
    BaseRule.__init__(self, cfn_model, debug=debug)

  def rule_text(self):
    """
    Get rule text
    :return: 
    """
    if self.debug:
      print('rule_text'+lineno())
    return 'IAM role should not allow Allow+NotResource'


  def rule_type(self):
    """
    Get rule type
    :return: 
    """
    self.type= 'VIOLATION::WARNING'
    return 'VIOLATION::WARNING'


  def rule_id(self):
    """
    Get rule id
    :return: 
    """
    if self.debug:
      print('rule_id'+lineno())
    self.id ='W21'
    return 'W21'


  def audit_impl(self):
    """
    Audit
    :return: violations 
    """
    if self.debug:
      print('IamRoleNotResourceOnPermissionPolicyRule - audit_impl'+lineno())

      print('vars: '+str(vars(self)))
      print('dir: '+str(dir(self)))

    violating_roles = []

    resources = self.cfn_model.resources_by_type('AWS::IAM::Role')

    if len(resources)>0:
      for resource in resources:
          if self.debug:
            print('resource: '+str(resource)+lineno())

          if hasattr(resource, 'policy_objects'):
            if self.debug:
              print('has policy obects ' + lineno())

            for policy in resource.policy_objects:

              if policy.policy_document.allows_not_resource():
                if self.debug:
                  print('has allows not resources')

                violating_roles.append(str(resource.logical_resource_id))
    else:
      if self.debug:
        print('no violating_roles' + lineno())

    if self.debug:
      print('returning violating_roles to'+lineno())

    if self.debug:
      input('Press enter to continue: '+lineno())

    return violating_roles


from __future__ import absolute_import, division, print_function
from terraform_model.model.ModelElement import ModelElement


class IAMPolicy(ModelElement):
    """
    IAM policy model
    """
    def __init__(self, cfn_model):
        """
        Initialize
        :param cfn_model: 
        """
        ModelElement.__init__(self, cfn_model)

        self.groups= []
        self.roles= []
        self.users= []
        self.resource_type = 'AWS::IAM::Policy'






from __future__ import absolute_import, division, print_function
import sys
class Serverless:
    """
    Serverless
    """

    def __init__(self, debug=False):
        '''
        Initialize
        :param debug:
        '''
        self.debug = debug

        if self.debug:
            print('init')

    def perform_transform(self, cfn_hash):
        '''
        ???
        :param cfn_hash:
        :return:
        '''
        if self.debug:
            print('perform_transform')
            print('cfn_hash: '+str(cfn_hash))
        resources = {}
        sys.exit(1)
        resources = cfn_hash['Resources']
        if self.debug:
            print('resources: '+str(resources))
        # resources = cfn_hash['Resources'].clone

        for resource in resources:
            if self.debug:
                print(resource)
        # resources.each do |resource_name, resource|
        #  next unless resource['Type'].eql? 'AWS::Serverless::Function'
        #  replace_serverless_function cfn_hash, resource_name



    def instance(self):
          '''
          ???
          :return:
          '''
          if self.debug:
            print('instance')
          # FIXME
          sys.exit(1)

    def bucket_from_uri(self, uri):
          '''
          ???
          :param uri:
          :return:
          '''
          if self.debug:
            print('bucket_from_uri')
          # FIXME
          sys.exit(1)

    def object_key_from_uri(self, uri):
          '''
          ???
          :param uri:
          :return:
          '''
          if self.debug:
            print('object_key_from_uri')
          sys.exit(1)

    def is_s3_uri(self, uri):
          '''
          ???
          :param uri:
          :return:
          '''
          if self.debug:
            print('is_s3_uri')

          sys.exit(1)

    def replace_serverless_function(self, cfn_hash, resource_name):
          '''
          ???
          :param cfn_hash:
          :param resource_name:
          :return:
          '''
          if self.debug:
            print('replace_serverless_function')
          # FIXME
          sys.exit(1)
          # resource = cfn_hash['Resources'][resource_name]
          # code_uri = resource['Properties']['CodeUri']

          # lambda_fn_params = {
          #  handler: resource['Properties']['Handler'],
          #  runtime: resource['Properties']['Runtime']
          # }
          # if is_s3_uri? code_uri
          #  lambda_fn_params[:code_bucket] = bucket_from_uri code_uri
          #  lambda_fn_params[:code_key] = object_key_from_uri code_uri
          # end
          # cfn_hash['Resources'][resource_name] = \
          #  lambda_function lambda_fn_params

          # cfn_hash['Resources']['FunctionNameRole'] = function_name_role

    def function_name_role(self):
          '''
          ???
          :return:
          '''
          if self.debug:
            print('function_name_role')
          # FIXME
          sys.exit(1)
          #  {
          #    'Type' => 'AWS::IAM::Role',
          #    'Properties' => {
          #      'ManagedPolicyArns' =>
          #      ['arn:aws:iam::aws:policy/service-role/' \
          #        'AWSLambdaBasicExecutionRole'],
          #      'AssumeRolePolicyDocument' => {
          #        'Version' => '2012-10-17',
          #        'Statement' => [{
          #          'Action' => ['sts:AssumeRole'], 'Effect' => 'Allow',
          #          'Principal' => { 'Service' => ['lambda.amazonaws.com'] }
          #        }]
          #      }
          #    }
          #  }

    def lambda_function(self, handler, runtime, code_bucket=None, code_key=None):
          '''
          ???
          :param handler:
          :param runtime:
          :param code_bucket:
          :param code_key:
          :return:
          '''
          if self.debug:
            print('lambda_function')
          # FIXME
          sys.exit(1)
          # fn_resource = \
          #    {'Type' = > 'AWS::Lambda::Function',
          #                'Properties' = > {
          #    'Handler' = > handler,
          #                  'Role' = > {'Fn::GetAtt' = > % w[FunctionNameRole
          # Arn]},
          # 'Runtime' = > runtime
          # }}
          # if code_bucket & & code_key
          #    fn_resource['Properties']['Code'] = {
          #        'S3Bucket' = > code_bucket,
          #                       'S3Key' = > code_key
          #    }
          #    end


from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule
from  terraform_model.model.LambdaPrincipal import LambdaPrincipal

def lineno():
    """Returns the current line number in our program."""
    return str(' - LambdaPermissionWildcardPrincipalRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))



class LambdaPermissionWildcardPrincipalRule(BaseRule):
  

  def __init__(self, cfn_model=None, debug=None):
    """
    Initialize
    :param cfn_model: 
    """
    BaseRule.__init__(self, cfn_model, debug=debug)

  def rule_text(self):
    """
    Get rule text
    :return: 
    """
    if self.debug:
      print('rule_text'+lineno())
    return 'Lambda permission principal should not be wildcard'


  def rule_type(self):
    """
    Get rule type
    :return: 
    """
    self.type= 'VIOLATION::FAILING_VIOLATION'
    return 'VIOLATION::FAILING_VIOLATION'

  def rule_id(self):
    """
    Get rule id
    :return: 
    """
    if self.debug:
      print('rule_id'+lineno())
    self.id ='F13'
    return 'F13'


  def audit_impl(self):
    """
    Audit
    :return: violations 
    """
    if self.debug:
      print('LambdaPermissionWildcardPrincipalRule - audit_impl'+lineno())
    
    violating_lambdas = []
    
    resources = self.cfn_model.resources_by_type('AWS::Lambda::Permission')

    if len(resources)>0:
      for resource in resources:
          if self.debug:
            print('resource: '+str(resource)+lineno())
            print('vars: '+str(vars(resource))+lineno())

          if hasattr(resource, 'principal'):
            if self.debug:
              print('has lambda principal ' + lineno())

              print(resource.principal)
            if LambdaPrincipal.wildcard(resource.principal):
              violating_lambdas.append(str(resource.logical_resource_id))

    else:
      if self.debug:
        print('no violating_lambda' + lineno())

    if self.debug:
      input('Press enter to continue: '+lineno())

    return violating_lambdas



from __future__ import absolute_import, division, print_function
import inspect
from terraform_model.transforms import Serverless


def lineno():
    """Returns the current line number in our program."""
    return str(' -  TransformRegistry- line number: '+str(inspect.currentframe().f_back.f_lineno))


class TransformRegistry:
    """
    Transform registry parser
    """
    def __init__(self, debug=False):
        """
        Initialize
        :param debug: 
        """
        #self.registry= {'AWS::Serverless-2016-10-31' = > CfnModel::Transforms::Serverless}
        self.registry = Serverless.Serverless(debug=debug)
        self.debug = debug

        if self.debug:
            print('TransformRegistry - init'+lineno())

    def perform_transforms(self,cfn_hash):
        if self.debug:
            print('TranformRegistry - perform_transforms'+lineno())

        if 'Transform' in cfn_hash:
            transform_name = cfn_hash['Transform']

            if transform_name:
                self.registry.perform_transform(cfn_hash)

        else:

            return cfn_hash

    def instance(self):
        if self.debug:
            print('TransformRegistry - instance'+lineno())
        #@instance| |= TransformRegistry.new
        #@instance
        # FIXME
        sys.exit(1)


from __future__ import absolute_import, division, print_function
import sys
import inspect
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule

def lineno():
    """Returns the current line number in our program."""
    return str(' - IamRoleWildcardResourceOnPermissionsPolicyRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))



class IamRoleWildcardResourceOnPermissionsPolicyRule(BaseRule):


  def __init__(self, cfn_model=None, debug=None):
    """
    Initialize
    :param cfn_model:
    """
    BaseRule.__init__(self, cfn_model, debug=debug)

  def rule_text(self):
    """
    Get rule text
    :return:
    """
    if self.debug:
      print('rule_text'+lineno())
    return 'IAM role should not allow * resource on its permissions policy'

  def rule_type(self):
    """
    Get rule type
    :return:
    """
    self.type= 'VIOLATION::WARNING'
    return 'VIOLATION::WARNING'


  def rule_id(self):
    """
    Get rule id
    :return:
    """
    if self.debug:
      print('rule_id'+lineno())
    self.id ='W11'
    return 'W11'


  def audit_impl(self):
    """
    Audit
    :return: violations
    """
    if self.debug:
      print('IamRoleWildcardResourceOnPermissionsPolicyRule - audit_impl'+lineno())

    dir(self.cfn_model)
    if self.debug:
      print('vars: '+str(vars(self))+lineno())
      print('dir: '+str(dir(self))+lineno())
      print('cfn_model: '+str(self.cfn_model)+lineno())

    violating_roles = []

    resources = self.cfn_model.resources_by_type('AWS::IAM::Role')

    if self.debug:
      print('IamRoleWildcardResourceOnPermissionsPolicyRule - audit_impl - violating_roles: '+str(violating_roles)+lineno())

    if len(resources)>0:
      for resource in resources:
        if self.debug:
          print('resources: ' + str(resource) + lineno())
          print('vars: ' + str(vars(resource)) + lineno())

        if hasattr(resource, 'policy_objects'):
          if self.debug:
            print("\n"+'resource has policy obects ' + lineno())
            print('vars:'+str(resource.policy_objects)+lineno())
            print("\n")

          if resource.policy_objects:

            for policy in resource.policy_objects:
              if self.debug:
                print('policy:'+str(policy))
                print('vars: '+str(vars(policy))+lineno())
              if hasattr(policy,'policy_document'):
                if self.debug:
                  print('has policy document attribute')
                if policy.policy_document:
                  if policy.policy_document.wildcard_allowed_resources():
                    if self.debug:
                      print('has wildcard allowed resources')

                    violating_roles.append(str(resource.logical_resource_id))

    else:
      if self.debug:
        print('no violating_roles'+lineno())

    if self.debug:
      input('Press enter to continue: '+lineno())

    return violating_roles


from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule

def lineno():
    """Returns the current line number in our program."""
    return str(' - RDSInstanceMasterUsernameRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))


# cfn_nag rules related to RDS Instance master username
class RDSInstanceMasterUsernameRule(BaseRule):

  def __init__(self, cfn_model=None, debug=None):
      """
      Initialize
      :param cfn_model:
      """
      BaseRule.__init__(self, cfn_model, debug=debug)
      
  def rule_text(self):
      """
      Get rule text
      :return:
      """
      if self.debug:
          print('rule_text'+lineno())
      return 'RDS instance master username must be Ref to NoEcho Parameter. Default credentials are not recommended'


  def rule_type(self):
      """
      Get rule type
      :return:
      """
      self.type= 'VIOLATION::FAILING_VIOLATION'
      return 'VIOLATION::FAILING_VIOLATION'

  def rule_id(self):
      """
      Get rule id
      :return:
      """
      if self.debug:
          print('rule_id'+lineno())
      self.id ='F24'
      return 'F24'



  def audit_impl(self):
      """
      # Warning: if somebody applies parameter values via JSON, this will compare
      # that....
      # probably shouldn't be doing that though -
      # if it's NoEcho there's a good reason
      # bother checking synthesized_value? that would be the indicator.....
      :return: violations
      """
      if self.debug:
          print('RDSInstanceMasterUsernameRule - audit_impl'+lineno())
    
      violating_rdsinstances = []

      resources = self.cfn_model.resources_by_type('AWS::RDS::DBInstance')

      if len(resources)>0:
          for resource in resources:
              if self.debug:
                  print('resource: '+str(resource)+lineno())
                  print('resource: '+str(vars(resource))+lineno())

              if hasattr(resource,'masterUsername'):
                  if self.debug:
                      print('has masterUsername resource')
                  if resource.masterUsername:

                      if not self.references_no_echo_parameter_without_default(self.cfn_model,resource.masterUsername):
                          violating_rdsinstances.append(str(resource.logical_resource_id))

                      if self.debug:
                          print('violating_rdsinstances: ' + str(violating_rdsinstances) + lineno())

      else:
          if self.debug:
              print('no violating_rdsinstances' + lineno())

      if self.debug:
          input('Press enter to continue: ' + lineno())

      return violating_rdsinstances

  def to_boolean(self, string):
    """
    Not sure why
    :param string:
    :return:
    """
    if self.debug:
      print('to_boolean'+lineno())

    # FIXME
    sys.exit(1)
    #if string.to_s.casecmp('true').zero?
    #  true
    #else
    #  false
    #end


  def references_no_echo_parameter_without_default(self, cfn_model, master_username):
    """
    No echo parameter without defaults
    :param cfn_model:
    :param master_username:
    :return:
    """
    if self.debug:
      print('references_no_ech_parameter_without_default'+lineno())
      print('cfn_model: '+str(vars(cfn_model))+lineno())
      print('master_username: '+str(master_username)+lineno())

    if type(master_username) == type(dict()):

      if 'Ref' in master_username:
        if hasattr(cfn_model,'parameters'):
          if str(master_username['Ref']) in cfn_model.parameters:
            parameter = cfn_model.parameters[master_username['Ref']]

            if hasattr(parameter,'NoEcho'):
              if self.debug:
                print('has noecho property: '+lineno())
              return True


    if self.debug:
      print('Does not have noecho property '+lineno())
    return False


from __future__ import absolute_import, division, print_function
import inspect
import sys
from terraform_model.parser.ParserError import ParserError


def lineno():
    """Returns the current line number in our program."""
    return str(' -  LoadBalancerV2Parser- line number: '+str(inspect.currentframe().f_back.f_lineno))

class LoadBalancerV2Parser:
    """
    Load balancer v2 parser
    """
    @staticmethod
    def parse(cfn_model, resource, debug=False):
        """
        Parse load balancer v2
        :param resource: 
        :param debug: 
        :return: 
        """

        if debug:
            print('parse'+lineno())
            print('resource: '+str(resource))
            print('subnects: '+str(resource.subnets))

        if resource.subnets and len(resource.subnets)<2:

            try:
                raise ParserError('"Load Balancer must have at least two subnets:'+str(resource.subnets), None,debug=self.debug)
            except ParserError as e:

                raise ParserError('"Load Balancer must have at least two subnets:'+str(resource.subnets), None,debug=self.debug)
            #    #sys.exit(exc)

        if type(resource.securityGroups) == type(list()):
            for sg in resource.securityGroups:
                cfn_model.find_security_group_by_group_id(sg)

        else:
            # er... list of ids or comma separated list.  just punt.
            resource.security_groups = []

        return resource




from __future__ import absolute_import, division, print_function
import inspect
from terraform_model.parser.Error import Error
from builtins import (str)

def lineno():
    """Returns the current line number in our program."""
    return str(' -  ParserError- line number: '+str(inspect.currentframe().f_back.f_lineno))

class ParserError(Error):
    """
    Parser error
    """
    def __init__(self,message, validation_errors="None", debug=False):
        """
        Initialize
        :param message: 
        :param validation_errors: 
        :param debug: 
        """
        self.debug = debug
        self.message = message
        self.errors = validation_errors
        if self.debug:
            print('ParserError - init'+lineno())

    def to_hash(self):
        """
        Convert to hash
        :return: 
        """
        if self.debug:
            print('to_hash '+lineno())
        hash = {}
        hash[self.message] = self.errors
        return hash

    def to_string(self):
        """
        Convert to string
        :return: 
        """
        if self.debug:
            print('to_string'+lineno())
        print()
        return str(self.message)+ ' : '+str(self.errors)



from __future__ import absolute_import, division, print_function
import re
import sys
from builtins import (str)
from terraform_model.parser import ParserError
import inspect
import json
from builtins import (str)

def lineno():
    """Returns the current line number in our program."""
    return str(' -  ReferenceValidator- line number: '+str(inspect.currentframe().f_back.f_lineno))


class ReferenceValidator:
    """
    Reference validator
    """

    def __init__(self, debug=False):
        """
        Initialize
        :param debug:
        """

        self.debug=debug
        self.references = {}
        self.variables = {}
        self.outputs = {}

        if self.debug:

            print("\n\n#############################################")
            print('Creating ReferenceValidator - __init__'+lineno())
            print("################################################\n\n")

    def unresolved_references(self, cloudformation_hash):
        """
        Iterate over the cloudformation template
        :param cloudformation_hash:
        :return:
        """
        if self.debug:
            print("\n\n#################################################")
            print('ReferenceValidator - finding unresolved_references'+lineno())
            print("####################################################\n\n")

        if type(cloudformation_hash) == type(str()):
            json_acceptable_string = cloudformation_hash.replace("'", "\"")
            cloudformation_hash= json.loads(json_acceptable_string)


        if 'variable' in cloudformation_hash:
            if self.debug:
                print('There are variables in the cloudformation template'+lineno())

            parameter_keys = []
            for key in cloudformation_hash['variable']:
                parameter_keys.append(key)
        else:
            parameter_keys = []

        if self.debug:
            if parameter_keys:
                print('variable keys are: '+str(parameter_keys)+lineno())
            else:
                print('there are no variables in the template: '+lineno())

        if self.debug:
            print('cloudformation hash type: '+str(type(cloudformation_hash))+lineno())


        resource_keys = list(cloudformation_hash['resource'].keys())

        if self.debug:
            print('resource keys are: '+str(resource_keys)+lineno())


            print("\n\n#####################################")
            print("Trying to find missing refs"+lineno())
            print("#########################################\n")

        new_list = parameter_keys + resource_keys

        missing_refs = self.all_references(cloudformation_hash)

        sys.exit(1)
        if self.debug:
            print('new_list: '+str(new_list)+lineno())
            print('missing_refs: '+str(missing_refs)+lineno())

        for x in new_list:

            if self.debug:
                print(str(x)+lineno())
            if missing_refs:
                if x in missing_refs:

                    missing_refs.remove(x)

        if self.debug:
            print('missing_refs: '+str(missing_refs)+lineno())

        return missing_refs


    def all_references(self, cloudformation_hash):
        """
        Iterate over all resources in the cloudformation template

        :param cloudformation_hash:
        :return:
        """
        if self.debug:
            print("\n\n#############################################")
            print('ReferenceValidator - all_references'+lineno())
            print('hash: '+str(cloudformation_hash)+lineno())
            print("###############################################\n")


        result = None

        for resource in cloudformation_hash['resource']:

            if self.debug:
                print("\n\n###################################")
                print('resource type: '+str(resource)+lineno())
                print("#######################################\n")



            for resource_name in cloudformation_hash['resource'][resource]:
                if self.debug:
                    print('resource name: '+str(resource_name)+lineno())
                    print('resource attributes: '+str(cloudformation_hash['resource'][resource][resource_name].keys())+lineno())



                result_att = self.all_ref(cloudformation_hash['resource'][resource][resource_name])
                if self.debug:
                    print('result = '+str(result)+lineno())
                #result_att = self.all_get_att(cloudformation_hash['resource'][resource])

                #result_att = cloudformation_hash['resource'][resource].keys()


                if self.debug:
                    print('referense attributes are: '+str(result_att)+lineno())

                for item in result_att:
                    result.append(item)

                if self.debug:
                    print('result = '+str(result)+lineno())

            else:
                if self.debug:
                    print('resource: '+str(resource)+' has no properties'+lineno())

        if self.debug:
            print('returning result: '+str(result)+lineno())

        return result

    def all_ref(self, properties_hash):
        """
        Iterate over all properites in a resource and get all reference
        :param properties_hash:
        :return:
        """
        if self.debug:
            print("\n\n##############################################")
            print('ReferenceValidator - all_ref'+lineno())
            print('properties_hash: '+str(properties_hash)+lineno())
            print("##################################################\n")

        if type(properties_hash) == type(str()):
            json_acceptable_string = properties_hash.replace("'", "\"")
            properties_hash= json.loads(json_acceptable_string)



        if self.debug:
            print('type: '+str(type(properties_hash))+lineno())

        refs = []

        for property in properties_hash:

            if self.debug:
                print("\n########################################################")
                print('properties: '+str(properties_hash)+lineno())
                print('data type: '+str(type(properties_hash[property]))+lineno())
                print('resource_name: '+str(property)+lineno())
                print('value: '+str(properties_hash[property])+lineno())
                print("########################################################\n")

            if type(properties_hash[property]) == type(dict()):
                if self.debug:
                    print('is a dict: '+lineno())
                    print('property: '+str(property)+lineno())
                    print('value: '+str(properties_hash[property])+lineno())
                    print('subhash length: '+str(len(properties_hash[property]))+lineno())

                if str(properties_hash[property]).startswith('${'):
                    refs.append(properties_hash[property])

                #sub_hash = properties_hash[property]

                #if len(sub_hash) == 1 and 'Ref' in sub_hash:
                #    if self.debug:
                #        print('subhash hash length of one and Ref key'+lineno())
                #        print('subhash ref is: '+str(sub_hash['Ref'])+lineno())
                #        print('subhash ref type is: '+str(type(sub_hash['Ref']))+lineno())

                #    if type(sub_hash['Ref']) != type(str()) and type(sub_hash['Ref']) != type(unicode()):
                #        raise ParserError.ParserError('Ref target must be string literal: ' + str(sub_hash['Ref']),debug=self.debug)

                #    if not self.pseudo_reference(sub_hash['Ref']):
                #        refs.append(sub_hash['Ref'])
                #else:
                #    if self.debug:
                #        print('Trying to run all_refs again on subhash:'+str(sub_hash)+lineno())

                #    new_refs = self.all_ref(sub_hash)
                #    if self.debug:
                #        print('new refs: '+str(new_refs)+lineno())

                #    if new_refs:
                #        refs.extend(new_refs)
            else:
                if self.debug:
                    print('is not a dictionary: '+lineno())

                if str(properties_hash[property]).startswith('${'):
                    refs.append(properties_hash[property])


        if self.debug:
            print('refs: '+str(list(set(refs)))+lineno())

        input("Press Enter to continue..."+lineno())

        return list(set(refs))



    def all_get_att(self, properties_hash):
        """
        Iterate of all properties in hash
        :param properties_hash:
        :return:
        """
        if self.debug:
            print('ReferenceValidator - all_get_att'+lineno())
            print('properties_hash: '+str(properties_hash)+lineno())
        refs = ()

        if 'Properties' in properties_hash:
            if self.debug:
                print('properties hash ahs properties')

            # Iterating over all properties
            for property in properties_hash['Properties']:
                if self.debug:
                    print('property: '+str(property)+lineno())

                # If the property is a dictionary
                if type(properties_hash['variable'][property])== type(dict()):
                    if self.debug:
                        print('property is dict: '+lineno())

                    sub_hash = properties_hash['variable'][property]

                    # ! GetAtt too
                    if len(sub_hash)==1 and 'Fn::GetAtt' in sub_hash and sub_hash['Fn::GetAtt']==None:
                        if type(sub_hash['Fn::GetAtt']) == type(list()):
                            refs = refs + sub_hash['Fn::GetAtt'][0]
                        elif type(sub_hash['Fn::GetAtt']) == type(str()) or type(sub_hash['Fn::GetAtt']) == type(unicode()):
                            matchObj = re.match(r'([^.]*)\.(.*)', sub_hash['Fn::GetAtt'], re.M | re.I)

                            if matchObj:
                                if self.debug:
                                    print("matchObj.group() : ", matchObj.group())
                                    print("matchObj.group(1) : ", matchObj.group(1))
                                    print("matchObj.group(2) : ", matchObj.group(2))
                                refs = refs + matchObj.group(1)
                            else:
                                if self.debug:
                                    print("No match!!")
                    else:
                        refs = refs + self.all_get_att(sub_hash)
                else:
                    print('property is not a dictionary: '+lineno())
                    sys.exit(1)

        if self.debug:
            print('refs: '+str(refs)+lineno())

        return refs


    def pseudo_reference(self, ref):
        """
        Test whether the reference is a pseudo reference to AWS
        :param ref:
        :return:
        """
        if self.debug:
            print('ReferenceValidator - pseudo_reference')
            print('ref: '+str(ref))

        if 'AWS::' in ref:
            return True

        return False


from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)


def lineno():
    """Returns the current line number in our program."""
    return str(' - PolicyDocument - line number: '+str(inspect.currentframe().f_back.f_lineno))


class PolicyDocument:
    """
    Policy document model
    """

    def __init__(self, debug=False):
        """
        Initialize
        :param debug: 
        """
        self.statements = []
        self.version = None
        self.debug = debug

        if self.debug:
            print('init'+lineno())

    def wildcard_allowed_resources(self, debug=False):
        """
        Whether allow wildcard resources
        :param debug: 
        :return: 
        """
        if self.debug or debug:
            print('wilcard_allowed_resources'+lineno())

        for statement in self.statements:

            if len(statement.wildcard_resources())>0 and str(statement.effect) == 'Allow':
                return True

        return False


    def wildcard_allowed_actions(self, debug=False):
        """
        Wether wildcard allowed actions
        :param debug: 
        :return: 
        """
        if self.debug or debug:
            print('wildcard_allowed_actions'+lineno())

        for statement in self.statements:
            if self.debug or debug:

                print("\n\n#######################################################")
                print('statement in policy document: '+str(statement)+lineno())
                print('vars: '+str(vars(statement)))
                print('wildcard_actions:'+str(statement.wildcard_actions()))
                print("#######################################################\n")

            if str(statement.effect) == 'Allow':
                if self.debug or debug:
                    print('effect is to allow'+lineno())

                for action in statement.actions:
                    if self.debug or debug:
                        print('action: '+str(action)+lineno())
                        print('action type: '+str(type(action))+lineno())

                    if type(action) == type(str()):
                        if '*' in action:
                            return True

                    if sys.version_info[0] < 3 and type(action) == type(unicode()):
                            if '*' in str(action):
                                return True

                    elif type(action)== type(list()):
                        for action2 in action:
                            if self.debug or debug:
                                print('action: '+str(action2)+lineno())

                            if "*" in action2:
                                if self.debug or debug:
                                    print('* in action')
                                return True

        return False



    def wildcard_allowed_principals(self, debug=False):
        """
        Whether wildcard allowed principals
        :param debug: 
        :return: 
        """
        if self.debug or debug:
            print('wildcard_allowed_principals'+lineno())
            print('self: '+str(self)+lineno())
            print('vars: '+str(vars(self))+lineno())
            print('number of statements: '+str(len(self.statements))+lineno())

        for statement in self.statements:
            if self.debug or debug:
                print('statment: '+str(statement)+lineno())
                print('vars: '+str(vars(statement)))
                print('wildcard_principal:'+str(statement.wildcard_principal())+lineno())

            if str(statement.effect) == 'Allow':
                if self.debug or debug:
                    print('effect is to allow'+lineno())

                if statement.wildcard_principal():
                    return True

        return False


    def allows_not_action(self, debug=False):
        """
        Select any statement object that allow in conjunction with a NotAction
        :param debug: 
        :return: 
        """
        if self.debug or debug:
            print('allows_not_action'+lineno())

        for statement in self.statements:
            if self.debug or debug:
                print('statement: '+str(statement)+lineno())
                print('vars: '+str(vars(statement))+lineno())

            if len(statement.not_actions) > 0 and str(statement.effect) == 'Allow':
                if self.debug or debug:
                    print('has not actions and effect is allow'+lineno())
                return True

        return False



    def allows_not_resource(self, debug=False):
        """
        Allows not resource
        :param debug: 
        :return: 
        """
        if self.debug or debug:
            print('allows_not_resources'+lineno())

        for statement in self.statements:
            if self.debug or debug:
                print('statement: '+str(statement)+lineno())

            if type(statement)== type(dict()):

                if 'NotResource' in statement and 'Effect' in statement:
                    if len(statement['NotResource'])>0 and statement['Effect'] == 'Allow':
                        return True

            elif hasattr(statement,'not_resources') and hasattr(statement,'effect'):

                    if len(statement.not_resources)>0 and str(statement.effect) == 'Allow':
                        return True

        return False

    def allows_not_principal(self, debug=False):
      """
      Allows not principal
      :param debug: 
      :return: 
      """
      if self.debug or debug:
        print('allows_not_principal'+lineno())

      for statement in self.statements:
          if self.debug or debug:
              print('statement: ' + str(statement) + lineno())

          if type(statement) == type(dict()):

              if 'NotPrincipal' in statement and 'Effect' in statement:
                  if len(statement['NotPrincipal']) > 0 and statement['Effect'] == 'Allow':
                      return True

          elif hasattr(statement, 'not_principal') and hasattr(statement, 'effect'):
              if statement.not_principal and statement.effect:
                  if len(statement.not_principal) > 0 and str(statement.effect) == 'Allow':
                    return True

      return False


    def to_string(self):
        """
        Convert to string
        :return: 
        """
        if self.debug:
            print('to_s'+lineno())
        # FIXME
        sys.exit(1)
        #<< END

        #{
        #    version =  # {@version}
        #statements =
        # {@statements}
        #}


from __future__ import absolute_import, division, print_function
import copy
import sys
import inspect
import re
import sys
import json
from terraform_model.model.EC2SecurityGroupIngress import EC2SecurityGroupIngress
from terraform_model.model.EC2SecurityGroupEgress import EC2SecurityGroupEgress
from terraform_model.model.References import References
from builtins import (str)


def lineno():
    """Returns the current line number in our program."""
    return str(' -  SecurityGroupParser- line number: '+str(inspect.currentframe().f_back.f_lineno))

class SecurityGroupParser:
    """
    Security group parser
    """
    
    @staticmethod
    def parse(cfn_model, resource, debug=False):
        """
        Parse security group
        :param resource:
        :param debug:
        :return:
        """
        if debug:
            print('parse'+lineno())

        security_group = copy.copy(resource)
        security_group = SecurityGroupParser.objectify_egress(cfn_model, security_group, debug=debug)
        security_group = SecurityGroupParser.objectify_ingress(cfn_model, security_group, debug=debug)


        if debug:
            print("\n###############################")
            print('objectified egress and ingress'+lineno())
            print('staring to wire ingress rules to security group'+lineno())
            print("#################################\n")

        security_group = SecurityGroupParser.wire_ingress_rules_to_security_group(cfn_model, security_group, debug=debug)
        security_group = SecurityGroupParser.wire_egress_rules_to_security_group(cfn_model, security_group, debug=debug)


        return security_group

    @staticmethod
    def objectify_ingress(cfn_model, security_group, debug=False):
        """
        Objectivy ingress statment
        :param security_group:
        :param debug:
        :return:
        """
        if debug:
            print("\n\n#########################")
            print('objectify_ingress'+lineno())

        if security_group:
            if debug:
                print('there is a security group'+lineno())
                print('security group: '+str(security_group)+lineno())
                print('type: '+str(type(security_group))+lineno())
                print('vars: '+str(vars(security_group))+lineno())
                print('security group raw model: '+str(security_group.raw_model)+lineno())
                print('###########################\n\n')


            # If there is a ingress in properties
            if security_group.ingress:

                if debug:
                    print('has securitygroupingress property'+lineno())
                    print('type: '+str(type(security_group.ingress))+lineno())


                if type(security_group.ingress) == type(str()):
                    json_acceptable_string = security_group.ingress.replace("'", "\"")
                    security_group.ingress= json.loads(json_acceptable_string)


                if debug:
                    print('security group ingress is now: '+str(security_group.ingress)+lineno())

                # if the ingress is a list
                if type(security_group.ingress) == type(list()):
                    if debug:
                        print('is a list'+lineno())

                    if debug:
                        print('Iterating over each list in array'+lineno())
                    # Iterate over each sg in array
                    for sg in security_group.ingress:
                        if debug:
                            print(str(sg)+lineno())

                        ingress_object = EC2SecurityGroupIngress(cfn_model,debug=debug)
                        ingress_object.logical_resource_id = security_group.logical_resource_id

                        for key in sg:
                            if debug:
                                print('key: '+str(key)+lineno())
                                print('value: '+str(sg[key])+lineno())

                            if '::' in key:
                                if debug:
                                    print(':: in key'+lineno())
                                continue
                            else:
                                if debug:
                                    print(':: not in key '+lineno())
                                new_key_name = SecurityGroupParser.initialLower(key)

                                if debug:
                                    print('new key name: '+str(new_key_name)+lineno())
                                setattr(ingress_object,SecurityGroupParser.initialLower(key),sg[key])

                        if debug:
                            print(str(vars(ingress_object))+lineno())

                        security_group.ingresses.append(ingress_object)

                    if debug:
                        print('Done iterating of list - returning security group to caller '+lineno())

                    return security_group

                # If the ingress routes is a dictionary
                elif type(security_group.ingress) == type(dict()):

                    # example: security_group.ingress
                    # example: {'from_port': 0, 'to_port': 65535, 'protocol': 'TCP', 'cidr_blocks': ['${var.cidr_blocks}']}

                    if debug:
                        print('is a dict'+lineno())
                        print(str(security_group.ingress)+lineno())


                    ingress_object = EC2SecurityGroupIngress(cfn_model)
                    ingress_object.logical_resource_id = security_group.logical_resource_id

                    # Iterate over each key-value pair in dictionary and associate
                    # to an object attribute
                    for key in security_group.ingress:
                        if debug:
                            print('key: ' + str(key) + lineno())
                            print('value: ' + str(security_group.ingress[key]) + lineno())

                        setattr(ingress_object, SecurityGroupParser.initialLower(key), security_group.ingress[key])

                    security_group.ingresses.append(ingress_object)
                    return security_group

                # if the ingress is a list
                else:
                    print('security group ingress is not a list or dict')
                    print(str('type: ')+str(type(security_group.ingress)+lineno()))
                    sys.exit(1)

    @staticmethod
    def objectify_egress(cfn_model, security_group, debug=False):
        """
        Trying to convert a security group egress in to an egress object
        :param security_group:
        :param debug:
        :return:
        """
        if debug:
            print("\n\n###############")
            print('objectify_egress'+lineno())
            print('security group type: '+str(type(security_group))+lineno())
            print(str(vars(security_group))+lineno())
            print('##################\n\n')

        if security_group:
            if debug:
                print("\n\n#######################################")
                print('Details regarding security group')
                print('there is a security group'+lineno())
                print('security group: '+str(security_group)+lineno())
                print('vars: '+str(vars(security_group))+lineno())
                print('model: '+str(security_group.cfn_model)+lineno())
                print("###########################################\n")


            print(str(security_group.logical_resource_id)+lineno())
            print(str(security_group.egress)+lineno())
            if security_group.egress:
                print(str(security_group.egress)+lineno())


            # If there is a egress property
            if security_group.egress:

                if type(security_group.egress) == type(str()):
                    json_acceptable_string = security_group.egress.replace("'", "\"")
                    security_group.egress= json.loads(json_acceptable_string)

                if debug:
                    print('has securitygroupegress property'+lineno())
                    print('type: '+str(type(security_group.egress))+lineno())

                # If the egress is an array
                if type(security_group.egress) == type(list()):
                    if debug:
                        print('is a list'+lineno())

                    for sg in security_group.egress:
                        if debug:
                            print(str(sg)+lineno())

                        egress_object = EC2SecurityGroupEgress(cfn_model,debug=debug)
                        egress_object.logical_resource_id = security_group.logical_resource_id

                        for key in sg:
                            if debug:
                                print('key: '+str(key)+lineno())
                                print('value: '+str(sg[key])+lineno())

                            if '::' in key:
                                continue

                            else:
                                if debug:
                                    print(':: not in key '+lineno())
                                new_key_name = SecurityGroupParser.initialLower(key)

                                if debug:
                                    print('new key name: '+str(new_key_name)+lineno())

                                setattr(egress_object,SecurityGroupParser.initialLower(key),sg[key])

                        if debug:
                            print(str(vars(egress_object))+lineno())

                    security_group.egresses.append(egress_object)

                    return security_group

                # If the egress is a dictionary
                elif type(security_group.egress) == type(dict()):

                    if debug:
                        print('is a dict'+lineno())
                        print(str(security_group.egress)+lineno())
                    # {'CidrIp': '10.1.2.3/32', 'FromPort': 34, 'ToPort': 36, 'IpProtocol': 'tcp'}

                    egress_object = EC2SecurityGroupIngress(cfn_model)
                    egress_object.logical_resource_id = security_group.logical_resource_id

                    # Iterate over each key-value pair in the dictionary and create
                    # an object attribute
                    for key in security_group.egress:
                        if debug:
                            print('key: ' + str(key) + lineno())
                            print('value: ' + str(security_group.egress[key]) + lineno())

                        matchObj = re.match(r'::', key, re.M | re.I)

                        if matchObj:
                            if debug:
                                print("matchObj.group() : ", matchObj.group() + lineno())
                            continue
                        else:
                            if debug:
                                print("No match!!" + lineno())

                            setattr(egress_object, SecurityGroupParser.initialLower(key), security_group.egress[key])

                    if debug:
                        print(str(vars(egress_object)) + lineno())

                    security_group.egresses.append(egress_object)
                    return security_group

                else:
                    print("\n#############################")
                    print('Security group is not egress')
                    print("################################\n")

        else:
            if debug:
                print('no security group'+lineno())

        return security_group

    @staticmethod
    def initialLower(key_name):
        """
        First character to lower case
        :return:
        """
        first_character = str(key_name)[:1]
        remaining_characters = str(key_name)[1:]
        new_property_name = str(first_character.lower()) + str(remaining_characters)

        return new_property_name

    @staticmethod
    def wire_ingress_rules_to_security_group(cfn_model, security_group, debug=False):
        """
        Wires a standalone ingress rule to a security group
        :param security_group:
        :param debug:
        :return:
        """
        if debug:
            print("\n\n###############")
            print('wire_ingress_rules_to_security_group'+lineno())
            print('##################\n\n')

            print('sg: '+str(security_group)+lineno())
            print('cfn_model: '+str(cfn_model)+lineno())
            print('vars: '+str(vars(cfn_model))+lineno())

        if not security_group:

            if self.debug:
                print('there is not a security group returning'+lineno())

            return security_group

        # Get all the EC2::SecurityGroupIngress resources
        ingress_rules = cfn_model.resources_by_type('AWS::EC2::SecurityGroupIngress')

        # Iterate over each of the ingress resources
        for security_group_ingress in ingress_rules:

            if debug:
                print("\n\n###########################################################")
                print('Standalone ingress resource')
                print('security_group_ingress: '+str(security_group_ingress)+lineno())
                print('vars: '+str(vars(security_group_ingress))+lineno())
                print('dirs: '+str(dir(security_group_ingress))+lineno())
                if security_group.ingress:
                    print('ingress: '+str(security_groupingress.ingress)+lineno())
                print("##############################################################\n")

            if security_group_ingress.ingress:
                if debug:
                    print('security group ingress cfn model has properties'+lineno())
                if security_group_ingress.logical_resource_id:
                    if debug:
                        print('security group ingress has groupid '+str(security_group_ingress.logical_resource_id)+lineno())

                    group_id = References.resolve_security_group_id(security_group_ingress.logical_resource_id,debug=debug)
                    if debug:
                        print('group id: '+str(group_id)+lineno())
                        print('security group logical resource id: '+str(security_group_ingress.logical_resource_id)+lineno())

                    # standalone ingress rules are legal - referencing an external security group
                    if not group_id:
                        continue

                    # If the group id in the standalone ingress matches the logical resource id
                    # of the actual security group
                    if security_group_ingress.ingress:
                        security_group.ingresses.append(security_group_ingress)

                else:
                    print('Security group ingress has no groupid')
                    sys.exit(1)

        if debug:
            print('done wiring_ingress_rules_to_security_group '+lineno())

        return security_group

    @staticmethod
    def wire_egress_rules_to_security_group(cfn_model, security_group, debug=False):
        """
        Wire egress rule to a security group
        :param security_group:
        :param debug:
        :return:
        """
        if debug:
            print("\n\n###############")
            print('wire_egress_rules_to_security_group'+lineno())
            print('##################\n\n')
            print('sg: '+str(security_group)+lineno())
            print('cfn_model: '+str(cfn_model)+lineno())
            print('vars: '+str(vars(cfn_model))+lineno())

        if not security_group:
            return security_group
        else:
            if debug:
                print("\n#############################")
                print('There is a security group')
                print("###############################\n")

        #egress_rules = cfn_model.resources_by_type('AWS::EC2::SecurityGroupEgress')
        egress_rules = cfn_model.resources_by_type('AWS::EC2::SecurityGroup')

        if debug:
            print('egress rules: '+str(egress_rules)+lineno())

        # Iterate over each of the egress resources
        for security_group_egress in egress_rules:

            if debug:
                print("\n\n###########################################################")
                print('Standalone egress resource')
                print('security_group_egress: '+str(security_group_egress)+lineno())
                print('vars: '+str(vars(security_group_egress))+lineno())
                print('dirs: '+str(dir(security_group_egress))+lineno())
                if security_group.egress:
                    print('egress: '+str(security_group.egress)+lineno())
                print("##############################################################\n")

            if security_group_egress.egress:
                print('security group egress cfn model has properties'+lineno())

                if security_group_egress.logical_resource_id:
                    print('security group ingress has groupid '+str(security_group_egress.logical_resource_id)+lineno())

                    group_id = References.resolve_security_group_id(security_group_egress.logical_resource_id,debug=debug)
                    if debug:
                        print('group id: '+str(group_id)+lineno())
                        print('security group logical resource id: '+str(security_group_egress.logical_resource_id)+lineno())

                    # standalone egress rules are legal - referencing an external security group
                    if not group_id:
                        continue

                    # If the group id in the standalone egress matches the logical resource id
                    # of the actual security group
                    if security_group_egress.egress:
                        security_group.egresses.append(security_group_egress)


        return security_group


from __future__ import absolute_import, division, print_function
import inspect
from terraform_model.model.ModelElement import ModelElement

def lineno():
    """Returns the current line number in our program."""
    return str(' - Policy - line number: '+str(inspect.currentframe().f_back.f_lineno))


class Policy(ModelElement):
    """
    Policy model
    """

    def __init__(self, debug=False):
        """
        Initialize
        :param debug: 
        """
        self.name = None
        self.policy_document = None
        self.debug = debug

        if self.debug:
            print('__init__'+lineno())




import os
import unittest
import sys
from collections import OrderedDict
from terraform_validator.ValidateUtility import ValidateUtility as class_to_test


def pretty(value, htchar='\t', lfchar='\n', indent=0):

    nlch = lfchar + htchar * (indent + 1)
    if type(value) == type(dict()) or type(value) == type(OrderedDict()):
        items = [
            nlch + repr(key) + ': ' + pretty(value[key], htchar, lfchar, indent + 1)
            for key in value
        ]
        return '{%s}' % (','.join(items) + lfchar + htchar * indent)

    elif type(value) == type(list()):
        items = [
            nlch + pretty(item, htchar, lfchar, indent + 1)
            for item in value
        ]

        if items:
            items = sorted(items)
        [str(item) for item in items]
        return '[%s]' % (','.join(items) + lfchar + htchar * indent)

    elif type(value) is tuple:
        items = [
            nlch + pretty(item, htchar, lfchar, indent + 1)
            for item in value
        ]
        return '(%s)' % (','.join(items) + lfchar + htchar * indent)

    else:
        return repr(str(value))

class TestEbsVolume(unittest.TestCase):
    """
    Test ec2 instance
    """

    def test_ebs_volume_no_tags(self):

      expected_result =  [
            {
                "failure_count": "1",
                "filename": "/tf/ec2_volume/no_tags.tf",
                "file_results": [
                    {
                        "id": "F89",
                        "type": "VIOLATION::FAILING_VIOLATION",
                        "message": "Ebs volume does not have the required tags of Name, ResourceOwner, DeployedBy, Project",
                        "logical_resource_ids": ["NewVolume"]
                    }
                ]
            }
        ]

      if sys.version_info[0] < 3:
          new_file_results = []

          for info in expected_result[0]['file_results']:
              print('info: ' + str(info))
              print('type: ' + str(type(info)))
              order_of_keys = ["id", "type", "message", "logical_resource_ids"]

              new_results = OrderedDict()
              for key in order_of_keys:
                  new_results[key] = info[key]

              new_file_results.append(new_results)
              print('new file results: ' + str(new_file_results))

              expected_result[0]['file_results'] = new_file_results

          order_of_keys = ["failure_count", "filename", "file_results"]
          list_of_tuples = [(key, expected_result[0][key]) for key in order_of_keys]
          expected_result = [OrderedDict(list_of_tuples)]

      expected_result = pretty(expected_result)

      template_name = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))+'/terraform_validator/test_templates/tf/ec2_volume/no_tags.tf'
      debug = False

      config_dict = {}
      config_dict['template_file'] = template_name
      config_dict['debug'] = debug
      config_dict['profile'] = None
      config_dict['rules_directory'] = None
      config_dict['input_path'] = None
      config_dict['profile'] = None
      config_dict['allow_suppression'] = False
      config_dict['print_suppression'] = False
      config_dict['parameter_values_path'] = None
      config_dict['isolate_custom_rule_exceptions'] = None
      config_dict['use_optional_rules'] = True
      validator = class_to_test(config_dict)

      real_result =  validator.validate()
      self.maxDiff = None

      print('expected results: '+str(expected_result))
      print('real results: '+str(real_result))

      self.assertEqual(expected_result.replace('\'','"'), real_result)


    def test_ec2_volume_has_required_tags(self):

      expected_result =   [
            {
                'failure_count': '0',
                'filename': '/json/ec2_volume/has_required_tags.json',
                'file_results': [
                ]
            }
        ]

      if sys.version_info[0] < 3:
          new_file_results = []

          for info in expected_result[0]['file_results']:
              print('info: ' + str(info))
              print('type: ' + str(type(info)))
              order_of_keys = ["id", "type", "message", "logical_resource_ids"]

              new_results = OrderedDict()
              for key in order_of_keys:
                  new_results[key] = info[key]

              new_file_results.append(new_results)
              print('new file results: ' + str(new_file_results))

              expected_result[0]['file_results'] = new_file_results

          order_of_keys = ["failure_count", "filename", "file_results"]
          list_of_tuples = [(key, expected_result[0][key]) for key in order_of_keys]
          expected_result = [OrderedDict(list_of_tuples)]

      expected_result = pretty(expected_result)



      template_name = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))+'/terraform_validator/test_templates/tf/ec2_volume/has_required_tags.tf'
      debug = False

      config_dict = {}
      config_dict['template_file'] = template_name
      config_dict['debug'] = debug
      config_dict['profile'] = None
      config_dict['rules_directory'] = None
      config_dict['input_path'] = None
      config_dict['profile'] = None
      config_dict['allow_suppression'] = False
      config_dict['print_suppression'] = False
      config_dict['parameter_values_path'] = None
      config_dict['isolate_custom_rule_exceptions'] = None
      config_dict['use_optional_rules'] = True
      validator = class_to_test(config_dict)

      real_result =  validator.validate()
      self.maxDiff = None

      print('expected results: '+str(expected_result))
      print('real results: '+str(real_result))

      self.assertEqual(expected_result, real_result)

    def test_ebs_volume_missing_one_tags(self):

      expected_result =  [
            {
                "failure_count": "1",
                "filename": "/tf/ec2_volume/missing_one_required_tags.tf",
                "file_results": [
                    {
                        "id": "F89",
                        "type": "VIOLATION::FAILING_VIOLATION",
                        "message": "Ebs volume does not have the required tags of Name, ResourceOwner, DeployedBy, Project",
                        "logical_resource_ids": ["NewVolume"]
                    }
                ]
            }
        ]

      if sys.version_info[0] < 3:
          new_file_results = []

          for info in expected_result[0]['file_results']:
              print('info: ' + str(info))
              print('type: ' + str(type(info)))
              order_of_keys = ["id", "type", "message", "logical_resource_ids"]

              new_results = OrderedDict()
              for key in order_of_keys:
                  new_results[key] = info[key]

              new_file_results.append(new_results)
              print('new file results: ' + str(new_file_results))

              expected_result[0]['file_results'] = new_file_results

          order_of_keys = ["failure_count", "filename", "file_results"]
          list_of_tuples = [(key, expected_result[0][key]) for key in order_of_keys]
          expected_result = [OrderedDict(list_of_tuples)]

      expected_result = pretty(expected_result)

      template_name = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))+'/terraform_validator/test_templates/tf/ec2_volume/missing_one_required_tags.tf'
      debug =  True

      config_dict = {}
      config_dict['template_file'] = template_name
      config_dict['debug'] = debug
      config_dict['profile'] = None
      config_dict['rules_directory'] = None
      config_dict['input_path'] = None
      config_dict['profile'] = None
      config_dict['allow_suppression'] = False
      config_dict['print_suppression'] = False
      config_dict['parameter_values_path'] = None
      config_dict['isolate_custom_rule_exceptions'] = None
      config_dict['use_optional_rules'] = True
      validator = class_to_test(config_dict)

      real_result =  validator.validate()
      self.maxDiff = None

      print('expected results: '+str(expected_result))
      print('real results: '+str(real_result))

      self.assertEqual(expected_result, real_result)



from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule

def lineno():
    """Returns the current line number in our program."""
    return str(' - SecurityGroupMissingEgressRule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))


class SecurityGroupMissingEgressRule(BaseRule):

    def __init__(self, cfn_model=None, debug=None):
        """
        Initialize
        :param cfn_model:
        """
        BaseRule.__init__(self, cfn_model, debug=debug)

    def rule_text(self):
        """
        Get rule text
        :return:
        """
        if self.debug:
            print('rule_text'+lineno())
        return 'Missing egress rule means all traffic is allowed outbound.  Make this explicit if it is desired configuration'


    def rule_type(self):
        """
        Get rule type
        :return:
        """
        self.type= 'VIOLATION::FAILING_VIOLATION'
        return 'VIOLATION::FAILING_VIOLATION'

    def rule_id(self):
        """
        Get rule id
        :return:
        """
        if self.debug:
            print('rule_id'+lineno())
        self.id ='F1000'
        return 'F1000'


    def audit_impl(self):
        """
        Audit
        :return: violations
        """
        if self.debug:
            print('SecurityGroupMissingEgressRule - audit_impl'+lineno())
        logical_resource_ids = []

        for groups in self.cfn_model.security_groups():
            if self.debug:
                print('group: '+str(groups)+lineno())
                print('vars:'+str(vars(groups))+lineno())

            if hasattr(groups,'egresses'):
                  if len(groups.egresses)<1:

                    logical_resource_ids.append(str(groups.logical_resource_id))

        if self.debug:
            print('violations: '+str(list(set(logical_resource_ids)))+lineno())

        if self.debug:
            input('Press enter to continue: ' + lineno())

        return logical_resource_ids


from __future__ import absolute_import, division, print_function
from terraform_model.model.ModelElement import ModelElement


class EC2SecurityGroup(ModelElement):
    """
    Ec2 security group model
    """
    def __init__(self, cfn_model):
        """
        Initialize
        :param cfn_model: 
        """

        ModelElement.__init__(self, cfn_model)
        self.securityGroupIngress = []
        self.securityGroupEgress = []
        self.ingresses = []
        self.egresses = []
        self.tags = []

        self.resource_type = 'AWS::EC2::SecurityGroup'


from __future__ import absolute_import, division, print_function
import inspect
import sys
from builtins import (str)
from terraform_validator.custom_rules.BaseRule import BaseRule
from terraform_validator.IpAddr import IpAddr



def lineno():
    """Returns the current line number in our program."""
    return str(' - SecurityGroupIngressCidrNon32Rule - caller: '+str(inspect.stack()[1][3])+' - line number: '+str(inspect.currentframe().f_back.f_lineno))


class SecurityGroupIngressCidrNon32Rule(BaseRule):

    def __init__(self, cfn_model=None, debug=None):
        """
        Initialize
        :param cfn_model:
        """
        BaseRule.__init__(self, cfn_model, debug=debug)

    def rule_text(self):
        """
        Get rule text
        :return:
        """
        if self.debug:
          print('rule_text'+lineno())
        return 'Security Groups found with ingress cidr that is not /32'


    def rule_type(self):
        """
        Get rule type
        :return:
        """
        self.type= 'VIOLATION::WARNING'
        return 'VIOLATION::WARNING'


    def rule_id(self):
        """
        Get rule id
        :return:
        """
        if self.debug:
          print('rule_id'+lineno())
        self.id ='W9'
        return 'W9'


    def audit_impl(self):
        """
        Audit
        :return: violations
        """
        if self.debug:
            print('SecurityGroupIngressCidrNon32Rule - audit_impl'+lineno())
        logical_resource_ids = []

        # Iterate over each of the security groups in the cloudformation template
        for groups in self.cfn_model.security_groups():

            if self.debug:
                print('group: '+str(groups)+lineno())
                print('vars: '+str(vars(groups))+lineno())

            # If the security group has ingresses
            if hasattr(groups,'ingresses'):
                if len(groups.ingresses)>0:

                    has_invalid_cidr = False

                    # Iterate over each on the ingresses
                    for ingresses in groups.ingresses:

                        if self.debug:
                          print('ingresses: '+str(ingresses)+lineno())

                        if type(ingresses) == type(dict()):

                            if self.debug:
                                print('ingress is a dict'+lineno())

                            if IpAddr.ip4_cidr_range(ingresses,debug=self.debug)==True or IpAddr.ip6_cidr_range(ingresses,debug=self.debug):
                                if self.debug:
                                    print('ip4/6 address is /32 or /128' + lineno())

                            else:
                                if self.debug:
                                    print('ip4/6 address does not end with /32 or /128' + lineno())

                                if self.debug:
                                    print("\n\n##########################################################")
                                    print('Resource is not valid - appending to list')
                                    print('logical resource id: ' + str(groups.logical_resource_id) + lineno())
                                    print("#############################################################\n")
                                logical_resource_ids.append(str(groups.logical_resource_id))

                        elif type(ingresses) == type(list()):

                            if self.debug:
                                print("ingress is a list() "+lineno())

                            for item in ingresses:

                                if IpAddr.ip4_cidr_range(item,debug=self.debug):
                                    if self.debug:
                                        print('ip4/6 address is /32 or /128' + lineno())
                                    continue

                                if IpAddr.ip6_cidr_range(item,debug=self.debug):
                                    if self.debug:
                                        print('ip4/6 address is /32 or /128' + lineno())
                                    continue

                            if self.debug:
                                print("\n\n##########################################################")
                                print('Resource is not valid - appending to list')
                                print('logical resource id: ' + str(groups.logical_resource_id) + lineno())
                                print("#############################################################\n")
                            logical_resource_ids.append(str(groups.logical_resource_id))

                        else:

                            if self.debug:
                                print('vars: '+str(vars(ingresses))+lineno())
                                print('ingress is not a list or dict'+lineno())

                            if hasattr(ingresses, 'cidrIp'):
                                if self.debug:
                                    print('has cidrIp '+lineno())

                                if IpAddr.ip4_cidr_range(ingresses,debug=self.debug):

                                    if self.debug:
                                        print('ip4/6 address is /32 or /128' + lineno())

                                    continue

                            if hasattr(ingresses,'cidrIpv6'):

                                if self.debug:
                                    print('ip4/6 address is /32 or /128'+lineno())

                                if IpAddr.ip6_cidr_range(ingresses, debug=self.debug):
                                    if self.debug:
                                        print('ip4/6 address is /32 or /128' + lineno())

                                continue

                            if not hasattr(ingresses,'cidrIp') and not hasattr(ingresses,'cidrIpv6'):
                                if self.debug:
                                    print('does not have a cidr entry')
                                continue

                            if self.debug:
                                print("\n\n##########################################################")
                                print('Resource is not valid - appending to list')
                                print('logical resource id: ' + str(groups.logical_resource_id) + lineno())
                                print("#############################################################\n")
                            logical_resource_ids.append(str(groups.logical_resource_id))
            else:
              sys.exit(1)

        if self.debug:
            print('violations: '+str(list(set(logical_resource_ids)))+lineno())

        if self.debug:
            print('Getting all the standalone ingress resources')

        standalone_resources= self.cfn_model.standalone_ingress()

        # iterate over the routes
        for resource in standalone_resources:

            if self.debug:
                print("\n\n#########################################")
                print('standalone resource: ' + str(resource) + lineno())
                print('vars: ' + str(vars(resource)) + lineno())
                print('type: '+str(type(resource))+lineno())
                print("############################################\n")

            if hasattr(resource,'cidrIp'):

                if self.debug:
                    print('has cidrIp attributes'+lineno())

                if IpAddr.ip4_cidr_range(resource.cidrIp,debug=self.debug):
                    if self.debug:
                        print('ip4/6 address is /32 or /128' + lineno())
                    continue

                else:
                    if self.debug:
                        print('ip4/6 address does not end with /32 or /128' + lineno())
                    logical_resource_ids.append(resource.logical_resource_id)

            if hasattr(resource,'cidrIpv6'):
                if self.debug:
                    print('has cidrIpv6 attributes' + lineno())

                if  IpAddr.ip6_cidr_range(resource.cidrIpv6,debug=self.debug):
                    if self.debug:
                        print('ip4/6 address is /32 or /128' + lineno())
                    continue

                else:
                    if self.debug:
                        print('ip4/6 address does not end with /32 or /128' + lineno())
                    logical_resource_ids.append(resource.logical_resource_id)

        if self.debug:
            print('violations: '+str(list(set(logical_resource_ids)))+lineno())

        if self.debug:
            input('Press enter to continue: ' + lineno())

        return logical_resource_ids


from __future__ import absolute_import, division, print_function
import sys
import inspect
from terraform_model.parser.PolicyDocumentParser import PolicyDocumentParser
from terraform_model.model.Policy import Policy


def lineno():
    """Returns the current line number in our program."""
    return str(' -  IamRoleParser- line number: '+str(inspect.currentframe().f_back.f_lineno))

class IamRoleParser:
    """
    IAM Role Parser
    """
    
    @staticmethod
    def parse(cfn_model, resource, debug=False):
        """
        Parse iam role
        :param resource: 
        :param debug: 
        :return: 
        """
        if debug:
            print('IAMRoleParser - parse'+lineno())
            print('resource: '+str(resource)+lineno())
            print('debug is: '+str(debug)+lineno())
            print('cfn_model: '+str(cfn_model)+lineno())
            print('policies: '+str(resource.policies)+lineno())
            print('managed policy arn: '+str(resource.managedPolicyArns)+lineno())
            print('policy objects: '+str(resource.policy_objects)+lineno())
            print('assume role policy document: '+str(resource.assume_role_policy_document)+lineno())
            print('raw model: '+str(resource.raw_model)+lineno())

        iam_role = resource

        document_parser =  PolicyDocumentParser(debug=debug)


        if debug:
            print('iam_role: '+str(iam_role)+lineno())
            print('vars: '+str(vars(iam_role)))

        if iam_role.assume_role_policy:
            iam_role.assume_role_policy_document  =document_parser.parse(cfn_model, iam_role.assume_role_policy)


        for policy in iam_role.policies:
            if debug:
                print('policy: '+str(policy)+lineno())

            if 'PolicyName' in policy:
                if debug:
                    print('has policy name: '+str(policy['PolicyName']))

                new_policy = Policy(debug=debug)
                new_policy.policy_name = policy['PolicyName']
                new_policy.policy_document = document_parser.parse(policy['PolicyDocument'])
                iam_role.policy_objects.append(new_policy)

        return iam_role


from __future__ import absolute_import, division, print_function
from terraform_model.model.ModelElement import ModelElement


class ElasticLoadBalancingLoadBalancer(ModelElement):
    """
    Elastic load balance load balancer model
    """
    def __init__(self, cfn_model):
        """
        Initialize
        :param cfn_model: 
        """
        ModelElement.__init__(self, cfn_model)

        self.securityGroups = []
        self.security_groups = []
        self.subnets = []
        self.tags = []
        self.availabilityZones = []
        self.instances = []
        self.appCookieStickinessPolicy = []
        self.lBCookieStickinessPolicy = []
        self.policies = []
        self.listeners = []
        self.access_logs={}
        self.resource_type = 'AWS::ElasticLoadBalancing::LoadBalancer'


from __future__ import absolute_import, division, print_function
import inspect
import sys
import os
from terraform_validator.CustomRuleLoader import CustomRuleLoader
from terraform_validator.ProfileLoader import ProfileLoader
from terraform_model.parser.ParserError import ParserError
from terraform_validator.result_views.RulesView import RulesView


def lineno():
    """Returns the current line number in our program."""
    return str(' - RuleDumper - line number: '+str(inspect.currentframe().f_back.f_lineno))


class RuleDumper:
    """
    Rule Dumper
    """

    def __init__(self, profile_definition=None, rule_directory=None, debug=False):
        """
        Initialize RuleDumper
        :param profile_definition: 
        :param rule_directory: 
        """
        self.debug = debug
        self.rule_directory = rule_directory
        self.profile_definition = profile_definition

        if self.debug:
            print('RuleDumper - init'+lineno())

    def dump_rules(self):
        """
        Dump rules
        :return: 
        """

        custom_rule_loader = CustomRuleLoader(debug=self.debug)

        if self.rule_directory:
            custom_rule_loader.extra_rule_directory = self.rule_directory

        custom_rule_loader.rule_directory = str(os.path.dirname(os.path.realpath(__file__)) + '/custom_rules')

        rule_registry = custom_rule_loader.rule_definitions()

        profile = None

        if self.profile_definition:

            if self.debug:
                print('profile definition: '+str(self.profile_definition)+lineno())

            try:
                profile_loader = ProfileLoader(custom_rule_loader.rule_definitions(),debug=self.debug)

                profile = profile_loader.load(custom_rule_loader.rule_definitions())
                if self.debug:
                    print('rule ids in profile: '+str(profile.rule_ids)+lineno())

            except ParserError as e:
                print('Error'+str(e)+lineno())
                sys.exit(1)

        view = RulesView()
        rules_view = view.emit(rule_registry=rule_registry, profile=profile, debug=self.debug)

        if self.debug:
            print('rules_view: '+str(rules_view))
